# frozen_string_literal: true

class RateAttack
  class Cache
    attr_accessor :prefix

    def initialize
      self.store = ::Rails.cache if defined?(::Rails.cache)
      @prefix = 'rate_attack'
    end

    attr_reader :store

    def store=(store)
      @store = store
    end

    def count(unprefixed_key, period)
      key, expires_in = key_and_expiry(unprefixed_key, period)
      do_count(key, expires_in)
    end

    def read(unprefixed_key)
      store.read("#{prefix}:#{unprefixed_key}")
    end

    def write(unprefixed_key, value, expires_in)
      store.write("#{prefix}:#{unprefixed_key}", value, expires_in: expires_in)
    end

    private

    def do_count(key, expires_in)
      result = store.increment(key, 1, expires_in: expires_in)
      # NB: Some stores return nil when incrementing uninitialized values
      if result.nil?
        store.write(key, 1, expires_in: expires_in)
      end
      result || 1
    end
  end
end
